#include "defbf532.h"

#define OFFSET_( x ) ((x) & 0x0000FFFF)  
#define UPPER_( x ) (((x) >> 16) & 0x0000FFFF)  
#define LOWER_( x ) ((x) & 0x0000FFFF)

/************************L1 Memory***********************************************************************************************/
#define L1_DataBA	0xFF800000		//L1 Data Bank A
#define L1_DataBB	0xFF900000		//L1 Data Bank B	
#define L1_Code		0xFFA00000		//L1 Code SRAM
#define L1_Scratch_END 0XFFB00FFC	//L1 Scratch SRAM (end)
#define ASYNC_Bank0 0x20000000		//Async Bank 0
/********************************************************************************************************************************/

#define SYSMMR_BASE 0xFFC00000
#define COREMMR_BASE 0xFFE00000

#define size 0x1E0

#define SYSCFG_VALUE 0x30			//For Anamoly #22

.SECTION program;

/************************Code to Stay in Supervisor Mode*************************************************************************/
	r1 = SYSCFG_VALUE;
	SYSCFG = r1;					//To allow operations up to 600MHz.

	SP.H = UPPER_(L1_Scratch_END);
	SP.L = LOWER_(L1_Scratch_END);	//Set up supervisor stack in the end of L1 Scratch Memory
	
	P0.H = UPPER_(COREMMR_BASE);	// P0 Points to the beginning of CORE MMR Space
	P0.L = LOWER_(COREMMR_BASE);
	
	P1.H = UPPER_(SYSMMR_BASE);		// P1 Points to the beginning of SYSTEM MMR Space
	P1.L = LOWER_(SYSMMR_BASE);
	
	P2.L = START;
	P2.H = START;
	[P0+OFFSET_(EVT15)] = P2;		//Place the address of start code in IVG15 of EVT
	
	R0 = [P0+OFFSET_(IMASK)];		
	R1.L = OFFSET_(EVT_IVG15);
	[P0+OFFSET_(IMASK)] = R1;		//Set(enable) IVG15
	RAISE 15;						//Invoke IVG15 interrupt
	P2.L = WAIT_HERE;
	P2.H = WAIT_HERE;
	RETI = P2;						//Set Reset Return Address 
	RTI;							//Return from Reset Interrupt
WAIT_HERE:	               			//Wait here till IVG15 interrupt is processed
	JUMP WAIT_HERE;
/********************************************************************************************************************************/

/************************Test BMODE Pins*****************************************************************************************/
START:
	[--SP] = RETI;					//RETI must be pushed to enable interrupts.  Boot code in supervisor mode starts here.


	P3.H = loader_file;
	P3.L = loader_file;
	R4.H = UPPER_(size);
	R4.L = LOWER_(size);

	R0 = 0x0085(Z);
	W[P1+OFFSET_(SPI_BAUD)] = R0;		//set baud rate register

	R2 = 0x0;							//Initial address

	R0 = W[P1+OFFSET_(FIO_DIR)](Z);
	BITSET(R0,2);
	W[P1+OFFSET_(FIO_DIR)] = R0;
	
	R0 = W[P1+OFFSET_(FIO_FLAG_S)](Z);
	BITSET(R0,2);
	W[P1+OFFSET_(FIO_FLAG_S)] = R0;
	
	CALL READ_STATUS_REGISTER;

ADDRESS:
	R0 = 0x5C01;
	W[P1+OFFSET_(SPI_CTL)] = R0;		//enable SPI, nondma tx mode, 8bit
	
	R0 = W[P1+OFFSET_(FIO_FLAG_C)](Z);
	BITSET(R0,2);
	W[P1+OFFSET_(FIO_FLAG_C)] = R0;

	R3 = R2;
	CALL ADDRESS_DECODE;
	
	R0 = 0x82(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out command
	
	R0 = R3 >> 16;
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 23 - 16 --> reserved:reserved:PA11:PA6)
	
	R0 = R3 >> 8;
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 15 - 8 --> PA5:BA8)
	
	R0 = R3;
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 7 - 0 --> BA7:BA0)
	
	P0 = R7;
	LSETUP(Begin, End) LC0 = P0;
Begin:
	R0 = B[P3++](Z) ;
	CALL SINGLE_BYTE_TRANSFER;			//send out byte
End: NOP;

	CALL Disable_SPI;

	R2 = R2 + R7;
	CC = R2 < R4;
	IF CC JUMP ADDRESS;
	
receive_data:
	P3.H = received_data;
	P3.L = received_data;
	
	R0 = 0x5C01;
	W[P1+OFFSET_(SPI_CTL)] = R0;		//enable SPI, nondma tx mode, 8bit
	
	R0 = W[P1+OFFSET_(FIO_FLAG_C)](Z);
	BITSET(R0,2);
	W[P1+OFFSET_(FIO_FLAG_C)] = R0;
	
	R0 = 0x68(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out command
	
	R0 = 0x00(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 23 - 16 --> reserved:reserved:PA11:PA6)
	
	R0 = 0x00(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 15 - 8 --> PA5:BA8)

	R0 = 0x00(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out address byte (bits 7 - 0 --> BA7:BA0)

	R0 = 0x0(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out don't care byte
	
	R0 = 0x0(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out don't care byte
	
	R0 = 0x0(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out don't care byte
	
	R0 = 0x0(Z);
	CALL SINGLE_BYTE_TRANSFER;			//send out don't care byte
	
	LC0 = R4;
	LSETUP(Begin_Read, End_Read) LC0;
Begin_Read:
	R0 = 0x0(Z) ;
	CALL SINGLE_BYTE_TRANSFER;			//send out dummy byte
	B[P3++] = R0;						//read byte from SPI memory
End_Read: NOP;

	CALL Disable_SPI;

jump(0);


///////////////////////////////////////////////////////////////////////////////////////////////
// SPI Byte Transfer Routine
// The following sends out a byte stored in R0, check's the Status Register for single-word 
// transfer complete, and finally reads a byte from the SPI memory and stores it in R0.
///////////////////////////////////////////////////////////////////////////////////////////////
SINGLE_BYTE_TRANSFER:
	W[P1+OFFSET_(SPI_TDBR)] = R0;		//send out byte
	
RD_ST:		
	R0 = W[P1+OFFSET_(SPI_STAT)] (Z);
	CC = BITTST(R0,5);					//test bit 5 (RXS) of SPIST0 register to see if the RX Data Buffer is full, if 0 do test again
	IF !CC JUMP RD_ST;
	
	R0 = W[P1+OFFSET_(SPI_RDBR)] (Z);	//do read
	
	RTS;

	
///////////////////////////////////////////////////////////////////////////////////////////////
// The following routine converts a logical address to a page number and a byte number for the
// ATMEL SPI DataFlash.
// The AT45DB041 has 11 page bits and 9 byte bits (264 bytes/page), R5.H = 0x1
// The AT45DB081 has 12 page bits and 9 byte bits (264 bytes/page), R5.H = 0x1
// The AT45DB161 has 12 page bits and 10 byte bits (528 bytes/page),R5.H = 0x0
///////////////////////////////////////////////////////////////////////////////////////////////	
ADDRESS_DECODE:
	R7 = 0x108;							//264 bytes per page for AT45DB041 and AT45DB081
	
	CC = BITTST(R5, 16);
	IF CC JUMP ATMEL_041_081;
	
ATMEL_161:
	R7 = 0x210;							//528 bytes per page for AT45DB161
	
ATMEL_041_081:
	R0 = 0x0;							//R0 holds the page number
	
COMPARE:
	CC = R3 <= R7;						//Is the logical address less than bytes/page?
	IF CC JUMP LESS_THAN_PAGE;
	
ADD_PAGE_SUBTRACT:
	R0 += 1;							//if not, add 1 to page number
	R3 = R3 - R7;						//and subtract bytes/page from logical address
	JUMP COMPARE;						//compare again

LESS_THAN_PAGE:	
	CC = R3 == R7;						//if so, check if we have exactly 1 page
	IF !CC JUMP FINISHED_DECODING;
	R3 = 0x0;							//if exact page, byte number = 0
	R0 += 1;							//and add 1 to page number

FINISHED_DECODING:
	R0 = R0 << 10;						//left shift page number by 10 for AT45DB161
	CC = BITTST(R5,16);
	IF !CC JUMP ADDRESS_161;
	R0 = R0 >> 1;						//left shift page number by 9 for AT45DB041 and AT45DB081
	
ADDRESS_161:
	R3 = R0|R3;							//OR page number and byte number
	
	RTS;
	
	
///////////////////////////////////////////////////////////////////////////////////////////////
// The following routine reads the status register for ATMEL SPI DataFlash.  This is to
// determine which memory we have connected:
// The AT45DB041 has bits 5:2 set to 0111 (set R5.H = 0x1)
// The AT45DB081 has bits 5:2 set to 1001 (set R5.H = 0x1)
// The AT45DB161 has bits 5:2 set to 1011 (set R5.H = 0x0)
///////////////////////////////////////////////////////////////////////////////////////////////
READ_STATUS_REGISTER:
	[--SP] = RETS;						//Save return address onto stack

	R0 = 0x5C01;
	W[P1+OFFSET_(SPI_CTL)] = R0;		//enable SPI, nondma tx mode, 8bit
	
	R0 = 0x4;
	W[P1+OFFSET_(FIO_FLAG_C)] = R0;
	
	R0 = 0xD7;							//send out control word to read status register
	CALL SINGLE_BYTE_TRANSFER;
	R0 = 0x0;
	CALL SINGLE_BYTE_TRANSFER;
	
	R5.H = 0x0;
	R1 = 0x3C;
	R0 = R0 & R1;
	R1 = 0x2C;
	CC = R0 == R1;
	IF CC JUMP STATUS_DONE;
	R5.H = 0x1;
	
STATUS_DONE:
	Call Disable_SPI;
	
	RETS = [SP++];
	RTS;

Disable_SPI:	
	R0 = 0x0000;
	W[P1+OFFSET_(SPI_CTL)] = R0;		//disable SPI, nondma tx mode, 8bit	
	
	R0 = W[P1+OFFSET_(FIO_FLAG_S)](Z);
	BITSET(R0,2);
	W[P1+OFFSET_(FIO_FLAG_S)] = R0;

Delay_loop:
	P5.H = 0x1000;
	P5.L = 0x0000;	
	LSETUP(Delay, Delay) LC0 = P5;
Delay: NOP;

	RTS;	
	
.SECTION L1_data;
.byte loader_file[size] = "filename.ldr";
.byte received_data[size];
